const expect = @import("std").testing.expect;

/// A 3D vector.
pub const Vec3 = struct {
    x: f32,
    y: f32,
    z: f32,

    /// Creates a new vector with the given components.
    pub fn new(x: f32, y: f32, z: f32) Vec3 {
        return Vec3{ .x = x, .y = y, .z = z };
    }

    /// Returns the dot product of this vector and another.
    pub fn dot(self: Vec3, other: Vec3) f32 {
        return self.x * other.x + self.y * other.y + self.z * other.z;
    }

    /// Returns the cross product of this vector and another.
    pub fn cross(self: Vec3, other: Vec3) Vec3 {
        return Vec3{
            .x = self.y * other.z - self.z * other.y,
            .y = self.z * other.x - self.x * other.z,
            .z = self.x * other.y - self.y * other.x,
        };
    }

    // Returns true if this vector is equal to another.
    pub fn equals(self: *const Vec3, other: *const Vec3) bool {
        return self.x == other.x and self.y == other.y and self.z == other.z;
    }
};

test "Dot product" {
    const a = Vec3{ .x = 1, .y = 2, .z = 3 };
    const b = Vec3{ .x = 4, .y = 5, .z = 6 };
    try expect(a.dot(b) == 32);
}

test "Cross product" {
    const a = Vec3{ .x = 1, .y = 2, .z = 3 };
    const b = Vec3{ .x = 4, .y = 5, .z = 6 };
    try expect(a.cross(b).equals(&Vec3{ .x = -3, .y = 6, .z = -3 }));
}
